	--[[
Name: LibXML-1.0
Revision: $Rev: 2 $
Author(s): Humbedooh
Description: XML import library
Dependencies: LibStub
License: MIT License
]]

local LIBXML = "LibXML-1.0"
local LIBXML_MINOR = tonumber(("$Rev: 02 $"):match("(%d+)")) or 10000;
if not LibStub then error(LIBXML .. " requires LibStub.") end
local LibXML = LibStub:NewLibrary(LIBXML, LIBXML_MINOR)
if not LibXML then return end


function LibXML.parseargs(s)
  local arg = {}
  string.gsub(s, "([%w%-]+)=([\"'])(.-)%2", function (w, _, a)
    arg[w] = a
  end)
  string.gsub(s, "([%w%-]+)=([%w%.%-]+)", function (w, a)
    arg[w] = a
  end)
  return arg
end

function LibXML:Import(s)
  local stack = {}
  local top = {}
  table.insert(stack, top)
  local ni,c,class,args, empty
  local i, j = 1, 1
  while true do
    ni,j,c,class,args, empty = string.find(s, "<(%/?)([%w:]+)(.-)(%/?)>", i)
    if not ni then break end
    local text = string.sub(s, i, ni-1)
    if not string.find(text, "^%s*$") then
      table.insert(top, text)
    end
    if empty == "/" then  -- empty element tag
      table.insert(top, {class=class, args=LibXML.parseargs(args), empty=1})
    elseif c == "" then   -- start tag
      top = {class=class, args=LibXML.parseargs(args)}
      table.insert(stack, top)   -- new level
    else  -- end tag
      local toclose = table.remove(stack)  -- remove top
      top = stack[#stack]
      if #stack < 1 then
        error(LIBXML .. ": No closing tag found for "..class)
      end
      if toclose.class ~= class then
        error(LIBXML .. ": Trying to close "..toclose.class.." with "..class)
      end
      table.insert(top, toclose)
    end
    i = j+1
  end
  local text = string.sub(s, i)
  if not string.find(text, "^%s*$") then
    table.insert(stack[#stack], text)
  end
  if #stack > 1 then
    error(LIBXML .. ": XML ended without closing "..stack[stack.n].class)
  end
  return stack[1]
end
